package Test::Repocop::CLI::Report;

use strict;
use warnings;
use File::Basename;
#use Test::Repocop::Log;
use Test::Repocop::Common;
use Test::Repocop::Workdir; # for --by-srpm-name
use Test::Repocop::TestDB::Filter;
use Source::Shared::CLI;
use Test::Repocop::CLI;
use Test::Repocop::CLI::Container;
# --include-
use Test::Repocop::CLI::Datadir;

our $VERSION=0.002;

our @ISA=qw/Source::Shared::CLI/;
#push @Source::Shared::VERBOSE_CLASSES, __PACKAGE__;
push @Test::Repocop::CLI::Container::OPT_CLASSES, __PACKAGE__;

my (@excludetest, @includetest);
my ($repocop_arg_lastrun, $repocop_arg_by_src_name,$opt_include_distrotests);

sub print_longopt {
    print "
  Repocop report options:
	--repocop-report <level>	Filter by level. Default is $Repocop::arg::reportlevel.
	--repocop-report-exclude-test <test,...>	Filter by test
	--repocop-report-include-test <test,...>	Filter by test
	--repocop-report-include-distrotests include all distrotests in test filter
	--repocop-id-except 	<except> mode. Process all ids except given.
	--repocop-id-given 	<given> mode. Process given ids.
	--repocop-id-last-run 	Use ids from the last repocop collect.
	--repocop-id-by-src-name Assume input to be src names, no repocop ids.
";
}

our @LONGOPT=(
    	# embedded
	"repocop-report|repocop-report-level=s" => \$Repocop::arg::reportlevel,
	"repocop-report-et|repocop-report-exclude-test=s"  => \@excludetest,
	"repocop-report-it|repocop-report-include-test=s"  => \@includetest,
	"repocop-report-include-distrotests" => \$opt_include_distrotests,
	"repocop-id-except"  => \$Repocop::arg::except,
	"repocop-id-given"  => sub { $Repocop::arg::except=0 },
	"repocop-id-last-run|repocop-id-lastrun"  => \$repocop_arg_lastrun,
	"repocop-id-by-src-name!"  => \$repocop_arg_by_src_name,
    );

our @SHORT_OPTIONS=(
	# public
	"report-level|report=s" => \$Repocop::arg::reportlevel,
	"report-et|report-exclude-test=s"  => \@excludetest,
	"report-it|report-include-test=s"  => \@includetest,
	"report-include-distrotests" => \$opt_include_distrotests,
	"id-except|except"  => \$Repocop::arg::except,
	"id-given|given"  => sub { $Repocop::arg::except=0 },
	"id-last-run|last-run|lastrun"  => \$repocop_arg_lastrun,
	"id-by-src-name|by-src-name!"  => \$repocop_arg_by_src_name,
    );

push @Test::Repocop::CLI::LONGOPT, @SHORT_OPTIONS;

sub process_options {
    &Test::Repocop::Workdir::die_if_nothing_to_report();
    %Repocop::arg::EXCLUDETEST=map {$_=>1} map {split ','} @excludetest;
    %Repocop::arg::INCLUDETEST=map {$_=>1} map {split ','} @includetest;

    if ($opt_include_distrotests) {
	my @tests_dir=@Repocop::datadir::tests_dir;
	die "Internal error: empty tests_dir" unless @tests_dir;
	%Repocop::arg::INCLUDETEST= map {$_=>1} map {basename(dirname($_))}
	    &Test::Repocop::Common::glob_tests('distrotest', @tests_dir);
    }

    &Repocop::arg::__set_report_mask($Repocop::arg::reportlevel);

    if ($repocop_arg_lastrun) {
	my $repocop_lastrun_file="$repocop_metadatadir/last-run";
	die "last run results not found. internal error?\n" unless -e $repocop_lastrun_file;
	my $f;
	open METADATA, $repocop_lastrun_file || die $!;
	while ($f=<METADATA>) {
	    chomp $f;
	    $Repocop::arg::pkgtable{$f}=1 if $f;
	}
	close METADATA;
    }
    ## HACK! for purge only :(
    if ($repocop_arg_by_src_name) {
	require Test::Repocop::RPMdb;
	# note: too much prepare inside;
	my $dbh=&Test::Repocop::SQL::connect_db("$repocop_test_dbdir/rpm.db");
	my ($srcids, $pkgids)=&Test::Repocop::RPMdb::get_rpm_db_ids_by_srpm_name($dbh,\@ARGV);
	map {$Repocop::arg::pkgtable{$_}=1} @$srcids, @$pkgids;
	&Test::Repocop::SQL::disconnect_db($dbh);
	$#ARGV=-1;
    }
}

sub process_arguments {
    foreach my $rpmarg (@ARGV) {
	if ( -d "$rpmarg") {
	    open (RPMARGS_FILES, "ls -1 $rpmarg |egrep -x '[^.].*[.](rpm|spec)\$' |") || die $!;
	    my $f;
	    while ($f=<RPMARGS_FILES>) {
		chomp $f;
		&_add_file_to_repocop_arg_pkg("$rpmarg/$f") if -e "$rpmarg/$f";
	    }
	    close RPMARGS_FILES;
	} else {
	    &_add_file_to_repocop_arg_pkg($rpmarg);
	}
    }
}

sub _add_file_to_repocop_arg_pkg {
    foreach (@_) {
	push @Repocop::arg::pkglist, $_;
	s!^.*/!!; # basename
	s/\.(?:rpm|spec)$//;
	$Repocop::arg::pkgtable{$_}=1;
    }
}

1;
